
----------------------------------------------------------------
-- [v1.01.00] Localized Functions and Tables to speed things up
-- [v1.03.03] Get uiScale values for initialising font size
----------------------------------------------------------------
local _L = nUI_InfoPanel_MagePorts_L;
local uiScaling = GetCVar("useUiScale");
local uiScale = tonumber(GetCVar("uiScale"));

---------------------------------------------------
-- [v1.01.00] Initialise Variables
-- [v1.01.00] Alpha and FontSize Values made global
-- [v1.02.00] PortalButtons and TeleportButtons made global
-- [v1.02.01] Added option to not display text in portal window
-- [v1.03.03] Set Default fontsize based on UI Scaling in Options
---------------------------------------------------
MagePorts_Defaults = {};
MagePorts_Defaults.MaxRestockQty = 10;
MagePorts_Defaults.UpdateInterval = 30;
if ( uiScaling == 0 or uiScale == 1 ) then
	MagePorts_Defaults.FontSize = 10;
else
	MagePorts_Defaults.FontSize = 14;
end
MagePorts_Defaults.NotAvailableAlpha = 0.1;
MagePorts_Defaults.NotIdealAlpha = 0.5;
MagePorts_Defaults.AvailableAlpha = 1.0;
MagePorts_Defaults.DisplayText = true;

MagePorts = {};
MagePorts.Class = nil;
MagePorts.Frame = nil;
MagePorts.LastUpdate = 0;
MagePorts.PortalButtons = {};
MagePorts.TeleportButtons = {};
local PortalDB = {};
local TeleportDB = {};
local ReagentDB = {};




---------------------------------------------------
-- [v1.03.00] - Added Reagent Database for flexibility
-- [v1.03.00] - Max Quantity to be set to RestockQty by default
---------------------------------------------------
local function MagePorts_InitReagentDB()
	ReagentDB = {};
	table.insert(ReagentDB, { item = 17031, max = MagePorts_Defaults.MaxRestockQty, level = 20 } );	-- Teleport Runes
	table.insert(ReagentDB, { item = 17032, max = MagePorts_Defaults.MaxRestockQty, level = 35 } );	-- Portal Runes
end

---------------------------------------------------
-- [v1.01.00] Initialise Portal Spell Database
---------------------------------------------------
local function MagePorts_InitPortalDB()
	PortalDB = {};
	table.insert(PortalDB, { level = 40, quest = false, spell = 10059, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 40, quest = false, spell = 11416, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 40, quest = false, spell = 11417, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 40, quest = false, spell = 11418, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 50, quest = false, spell = 11419, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 50, quest = false, spell = 11420, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 40, quest = false, spell = 32266, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 40, quest = false, spell = 32267, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 65, quest = false, spell = 33691, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 65, quest = false, spell = 35717, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 35, quest = false, spell = 49360, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 35, quest = false, spell = 49361, faction = _L["HORDE"] } );
	table.insert(PortalDB, { level = 74, quest = false, spell = 53142, faction = _L["ALLIANCE"] } );
	table.insert(PortalDB, { level = 74, quest = false, spell = 53142, faction = _L["HORDE"] } );
end

---------------------------------------------------
-- [v1.01.00] Initialise Teleport Spell Deatabase
---------------------------------------------------
local function MagePorts_InitTeleportDB()	
	TeleportDB = {};
	table.insert(TeleportDB, { level = 20, quest = false, spell =  3561, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 20, quest = false, spell =  3562, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 20, quest = false, spell =  3563, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 30, quest = false, spell =  3565, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 30, quest = false, spell =  3566, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 20, quest = false, spell =  3567, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 20, quest = false, spell = 32271, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 20, quest = false, spell = 32272, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 60, quest = false, spell = 33690, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 60, quest = false, spell = 35715, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 35, quest = false, spell = 49358, faction = _L["HORDE"] } );
	table.insert(TeleportDB, { level = 35, quest = false, spell = 49359, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 71, quest = true, spell = 53140, faction = _L["ALLIANCE"] } );
	table.insert(TeleportDB, { level = 71, quest = true, spell = 53140, faction = _L["HORDE"] } );
end

------------------------------------
-- [v1.01.00] Create a Secure Button
-- [v1.01.01] Only update cooldown if one exists
-- [v1.01.01] Validate Reagent Test before Cooldown Test for availability tooltip.
-- [v1.02.00] Removed cooldown count from tooltip
-- [v1.03.03] Made Button Texture a part of the Button hierarchy so that it can be adjusted as required
-- [v1.03.03] Removed NormalTexture as it was redundant
------------------------------------
local function MagePorts_CreateSecureSpellButton(spellID, reagentID, level, quest)
	local SpellName, _, SpellIcon = GetSpellInfo(spellID);
	local Available = ( GetSpellInfo(SpellName) ~= nil );
	local ButtonName = "P_" .. spellID;
	local TextureName = ButtonName .. "_Texture";
		
	-- Activate the cooldown by showing the frame
	local function ActivateCooldown(self)
		local SpellName, _, SpellIcon = GetSpellInfo(spellID);
		local CooldownStart, CooldownDuration, CooldownEnabled = GetSpellCooldown(SpellName);
		if ( CooldownEnabled ) then
			if ( CooldownDuration > 0 ) then 
				self.Text:Show();
				self:Show();
			end
		end
	end
	-- Deactivate the Cooldown for this button by hiding the frame
	local function DeActivateCooldown(self)
		self.Text:Hide();
		self:Hide();
	end
	-- Update the cooldown counter
	local function UpdateCooldown(self)
		local SpellName, _, SpellIcon = GetSpellInfo(spellID);
		local TimeNow = GetTime();
		local CooldownStart, CooldownDuration, CooldownEnabled = GetSpellCooldown(SpellName);
		if ( not CooldownEnabled ) then DeActivateCooldown(self); return; end
		if ( CooldownDuration == 0 ) then DeActivateCooldown(self); return; end	
		local DiffTime = TimeNow - CooldownStart;
		local CooldownCounter = CooldownDuration - DiffTime;
		CooldownCounter = math.floor(CooldownCounter);
		if ( CooldownCounter > 0 ) then	
			self.Text:SetText(CooldownCounter);	
		else
			DeActivateCooldown(self);
		end
	end
	
	local function UpdateAlpha(self)
		local reagentName = select(2,GetItemInfo(reagentID)) or nil;
		local reagentCount = GetItemCount(reagentID) or 0;
		local Available = ( GetSpellInfo(SpellName) ~= nil );
		local Solo = false;
		if ( GetNumRaidMembers() > 0) then 
			Solo = false; 
		elseif ( GetNumPartyMembers() > 0 ) then 
			Solo = false; 
		else
			Solo = true;
		end
		if ( not Available or reagentCount == 0 ) then 
			self:SetAlpha(MagePorts_Defaults.NotAvailableAlpha); 
		else
			if (Solo) then 
				self:SetAlpha(MagePorts_Defaults.NotIdealAlpha); 
			else
				self:SetAlpha(MagePorts_Defaults.AvailableAlpha);
			end
		end
		Path, Size, Flags = self.Cooldown.Text:GetFont()
		self.Cooldown.Text:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		self.Cooldown.Text:SetAlpha(1.0);
	end

	-- Show a tooltip for the button
	local function ShowTooltip(self)
		local Available = ( GetSpellInfo(SpellName) ~= nil );
		local TimeNow = GetTime();
		local CooldownStart, CooldownDuration, CooldownEnabled = GetSpellCooldown(SpellName);
		local CooldownCounter = 0;
		if ( CooldownEnabled ) then	CooldownCounter = CooldownDuration - (TimeNow - CooldownStart); end
		CooldownCounter = math.floor(CooldownCounter);
		local reagentCount = GetItemCount(reagentID);
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR");
		GameTooltip:SetText(SpellName, 1.0, 1.0, 0.0 );
		GameTooltip:AddLine(_L["LEVEL"] .. level,1.0,1.0,1.0);
		if ( quest ) then GameTooltip:AddLine(_L["ATTAINED_VIA_QUEST"], 1.0,1.0,1.0); end
		if ( not Available ) then
			GameTooltip:AddLine(_L["UNKNOWN"],1.0,1.0,1.0);
		elseif ( reagentCount < 1 ) then
			GameTooltip:AddLine(_L["OUT_OF_REAGENTS"], 1.0,1.0,1.0);
		elseif ( CooldownDuration == 0 ) then
			GameTooltip:AddLine(_L["AVAILABLE"], 1.0,1.0,1.0);
		elseif ( CooldownEnabled and CooldownDuration ~= 0) then
			GameTooltip:AddLine(_L["ON_COOLDOWN"], 1.0,1.0,1.0);
		end
		GameTooltip:Show();		
	end
	
	-- Hide the buttons tooltip
	local function HideTooltip(self)
		GameTooltip:Hide();
	end
	
	-- Create the button and its children and attributes
	local Button = CreateFrame("Button", ButtonName, UIParent,"ActionButtonTemplate,SecureActionButtonTemplate");			
	Button:SetWidth(36);
	Button:SetHeight(36);			
	Button.Texture = Button:CreateTexture(TextureName,"OVERLAY");
	Button.Texture:SetAllPoints(Button);
	Button.Texture:SetTexture(SpellIcon);
	Button:SetNormalTexture( "" ); 
	Button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", add);
	Button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	Button:SetAttribute("type","spell");
	Button:SetAttribute("spell",SpellName);
	Button:SetScript("OnEnter",ShowTooltip);
	Button:SetScript("OnLeave",HideTooltip);
	Button:SetScript("OnEvent",UpdateAlpha);
	Button:RegisterEvent("PLAYER_LOGIN");
	Button:RegisterEvent("ITEM_PUSH");
	Button:RegisterEvent("BAG_UPDATE");
	Button:RegisterEvent("PARTY_CONVERTED_TO_RAID");
	Button:RegisterEvent("PARTY_INVITE_CANCEL");
	Button:RegisterEvent("PARTY_INVITE_REQUEST");
	
	-- Create the Cooldown frame to display the Cooldown Counter
	Button.Cooldown = CreateFrame("Frame", ButtonName.."_Cooldown", Button);
	Button.Cooldown:SetAllPoints(Button);
	Button.Cooldown.Text = Button.Cooldown:CreateFontString(ButtonName.."_Cooldown_Text", 'ARTWORK', "GameFontNormal");
	local Path, Size, Flags = Button.Cooldown.Text:GetFont()
	Button.Cooldown.Text:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
	Button.Cooldown.Text:SetAllPoints(Button.Cooldown);
	Button.Cooldown.Text:SetJustifyH( "CENTER" );
	Button.Cooldown.Text:SetJustifyV( "MIDDLE" );
	Button.Cooldown.Text:SetTextColor( 1, 1, 0, 1 );
	Button.Cooldown.Text:SetText("");  
	Button.Cooldown.Text:Show();
	Button.Cooldown:Hide();
	Button.Cooldown:SetScript("OnUpdate",UpdateCooldown);
	Button.Cooldown:SetScript("OnEvent",ActivateCooldown);
	Button.Cooldown:RegisterEvent("SPELL_UPDATE_COOLDOWN");
		
	return Button;
end

---------------------------------------------------
-- [v1.01.00] Initialise the Buttons
-- [v1.03.00] Use ReagentDB itemID
---------------------------------------------------
local function MagePorts_InitButtons(parent)
	MagePorts.PortalButtons = {};
	MagePorts.TeleportButtons = {};
	local Faction = string.lower(UnitFactionGroup("player"));

	-- First the Portals
	for i,v in pairs(PortalDB) do
		if ( string.find(v.faction,Faction) ) then
			local Button = MagePorts_CreateSecureSpellButton(v.spell,ReagentDB[2].item, v.level, v.quest);
			Button:SetParent(parent);
			table.insert(MagePorts.PortalButtons,Button);
		end
	end
	
	-- Now the Teleports
	for i,v in pairs(TeleportDB) do
		if ( string.find(v.faction,Faction) ) then
			local Button = MagePorts_CreateSecureSpellButton(v.spell,ReagentDB[1].item, v.level, v.quest);
			Button:SetParent(parent);
			table.insert(MagePorts.TeleportButtons,Button);
		end
	end
end

---------------------------------------------------------------------------
-- [v1.01.00] Validate the Spell in case you cast the wrong one by mistake
-- [v1.01.02] Take into account casts that do not have spell names, links and ids
---------------------------------------------------------------------------
local function MagePorts_ValidateSpell(StartStop, Caster, SpellName, SpellRank)

	-- Not the player so don't proceed		
	if ( Caster ~= "player" ) then return false; end
	
	-- Not a spell then don't proceed
	if ( not SpellName ) then return false; end
	
	local SpellID = nil;
	local SpellLink = GetSpellLink(SpellName,"");

	-- No spell link to discern ID from so not a true spell
	if ( not SpellLink ) then return false; end	
	
	local Found, _, SpellString = string.find(SpellLink, "^|c%x+|H(.+)|h%[.*%]");
	if ( Found ) then _, SpellID = strsplit(":", SpellString); end
	SpellID = tonumber(SpellID);
	
	-- SpellID cannot be found so unable to validate spell
	if ( not SpellID ) then return false; end

	local PortalSpell = false;
	local TeleportSpell = false;
	for i,v in pairs(PortalDB) do
		v.spell = tonumber(v.spell);
		if ( v.spell == SpellID ) then PortalSpell = true; break; end
	end
	for i,v in pairs(TeleportDB) do
		v.spell = tonumber(v.spell);
		if ( v.spell == SpellID ) then TeleportSpell = true; break; end
	end	
	
	if ( StartStop == "STOP" ) then 
		if (PortalSpell or TeleportSpell) then return true; end
		return false;
	end
	
	if ( GetNumRaidMembers() > 0 ) then
		if ( TeleportSpell ) then UIErrorsFrame:AddMessage(_L["RAID_TELEPORT"], 1.0,1.0,0.0); end
	elseif ( GetNumPartyMembers() > 0 ) then
		if ( TeleportSpell ) then UIErrorsFrame:AddMessage(_L["PARTY_TELEPORT"], 1.0,1.0,0.0); end
	else
		if ( PortalSpell ) then UIErrorsFrame:AddMessage(_L["SOLO_PORTAL"], 1.0,1.0,0.0); end
	end
		
	if (PortalSpell or TeleportSpell) then return true; end
	return false;
end

---------------------------------------------------
-- [v1.01.00] Update the Frame with current information
-- [v1.02.00] Use Not Available alpha if no more reagents
-- [v1.02.01] Added option to not display text in portal window
-- [v1.02.02] Added ReagentName variable for those that don't know about it yet
-- [v1.03.00] Use ReagentDB data
-- [v1.03.03] Font Size now affects titles and counts
---------------------------------------------------
local function MagePorts_UpdateFrame(ForceUpdate)

	local Solo = false;
	local IsInRaid = false;
	local IsInParty = false;
	if ( GetNumRaidMembers() > 0) then 
		IsInRaid = true; 
	elseif ( GetNumPartyMembers() > 0 ) then 
		IsInParty = true; 
	else
		Solo = true;
	end
--[[
	local Path, Size, Flags;
	if ( MagePorts_Defaults.DisplayText  and reagentName ) then
		Path, Size, Flags = MagePorts.Frame.PortalCount:GetFont()
		MagePorts.Frame.PortalCount:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		MagePorts.Frame.PortalCount:SetText(reagentName .. " : " .. reagentCount);  
		Path, Size, Flags = MagePorts.Frame.PortalTitle:GetFont()
		MagePorts.Frame.PortalTitle:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		MagePorts.Frame.PortalTitle:SetText(_L["PORTALS"]);  
		Path, Size, Flags = MagePorts.Frame.TeleportTitle:GetFont()
		MagePorts.Frame.TeleportTitle:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		MagePorts.Frame.TeleportTitle:SetText(_L["TELEPORTS"]);  
		Path, Size, Flags = MagePorts.Frame.TeleportCount:GetFont()
		MagePorts.Frame.TeleportCount:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		MagePorts.Frame.TeleportCount:SetText(reagentName .. " : " .. reagentCount);  
	else
		MagePorts.Frame.PortalCount:SetText("");
		MagePorts.Frame.PortalTitle:SetText("");
		MagePorts.Frame.TeleportCount:SetText("");
		MagePorts.Frame.TeleportTitle:SetText("");
	end
--]]	
	-- First the Portal Buttons	
	for i,v in pairs(MagePorts.PortalButtons) do
		local reagentName = select(2,GetItemInfo(ReagentDB[2].item)) or nil;
		local reagentCount = GetItemCount(ReagentDB[2].item) or 0;
		local spellName = v:GetAttribute("spell");
		local Available = ( GetSpellInfo(spellName) ~= nil );
		if ( not Available or reagentCount == 0 ) then 
			v:SetAlpha(MagePorts_Defaults.NotAvailableAlpha); 
		else
			if (Solo) then 
				v:SetAlpha(MagePorts_Defaults.NotIdealAlpha); 
			else
				v:SetAlpha(MagePorts_Defaults.AvailableAlpha);
			end
		end
		Path, Size, Flags = v.Cooldown.Text:GetFont()
		v.Cooldown.Text:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		v.Cooldown.Text:SetAlpha(1.0);
	end

	Sized = false;

	-- Then the Teleport Buttons
	for i,v in pairs(MagePorts.TeleportButtons) do
		local reagentName = select(2,GetItemInfo(ReagentDB[1].item)) or nil;
		local reagentCount = GetItemCount(ReagentDB[1].item) or 0;
		local spellName = v:GetAttribute("spell");
		local Available = ( GetSpellInfo(spellName) ~= nil );
		if ( not Available or reagentCount == 0 ) then 
			v:SetAlpha(MagePorts_Defaults.NotAvailableAlpha); 
		else
			if (Solo) then 
				v:SetAlpha(MagePorts_Defaults.AvailableAlpha);
			else 
				v:SetAlpha(MagePorts_Defaults.NotIdealAlpha);
			end
		end
	    	Path, Size, Flags = v.Cooldown.Text:GetFont()
		v.Cooldown.Text:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
		v.Cooldown.Text:SetAlpha(1.0);
	end
	
end

---------------------------------------------------
-- [v1.01.00] Initialise the whole Frame
-- [v1.02.00] Removed Anchoring here while in nUI panel
-- [v1.02.01] Added option to not display text in portal window
-- [v1.03.03] Adjusted Position of Titles and Counts to just above the buttons they relate to.
---------------------------------------------------
local function MagePorts_InitFrame()
	
	-- Create the Frame itself
	local frame = CreateFrame("Frame","MagePorts_Frame",UIParent);
   	frame:SetWidth(250);        
   	frame:SetHeight(100);
   	frame:SetAlpha(1.0);        
   	frame:SetClampedToScreen(true);
	frame:SetPoint("CENTER",UIParent,"CENTER",0,0);
    
	-- Create the Buttons Table based on player faction
	MagePorts_InitButtons(frame);

	local cellCount = #MagePorts.PortalButtons;
	local cellSize = MagePorts.PortalButtons[1]:GetWidth();
	local cellGap = cellSize * 0.25;

	local function onPortalReagentEvent()
		local Path, Size, Flags;
		local reagentLink,reagentName,reagentCount;
		reagentName,reagentLink = GetItemInfo(ReagentDB[2].item);
		reagentCount = GetItemCount(ReagentDB[2].item) or 0;
		
		if ( MagePorts_Defaults.DisplayText and reagentLink ) then
			Path, Size, Flags = frame.PortalCount:GetFont()
			frame.PortalCount:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
			frame.PortalCount:SetText(reagentLink .. " : " .. reagentCount);  
			Path, Size, Flags = frame.PortalTitle:GetFont()
			frame.PortalTitle:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
			frame.PortalTitle:SetText(_L["PORTALS"]);  
		else
			frame.PortalCount:SetText("");
			frame.PortalTitle:SetText("");
		end
	end	
	local function onTeleportReagentEvent()
		local reagentLink,reagentName,reagentCount;
		local Path, Size, Flags;
		
		reagentName,reagentLink = GetItemInfo(ReagentDB[1].item);
		reagentCount = GetItemCount(ReagentDB[1].item) or 0;
		
		if ( MagePorts_Defaults.DisplayText and reagentLink) then
			Path, Size, Flags = frame.TeleportTitle:GetFont()
			frame.TeleportTitle:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
			frame.TeleportTitle:SetText(_L["TELEPORTS"]);  
			Path, Size, Flags = frame.TeleportCount:GetFont()
			frame.TeleportCount:SetFont(Path,MagePorts_Defaults.FontSize,Flags);
			frame.TeleportCount:SetText(reagentLink .. " : " .. reagentCount);  
		else
			frame.TeleportCount:SetText("");
			frame.TeleportTitle:SetText("");
		end
		
	end
	
	frame.PortalHeading = CreateFrame("Frame","MagePorts_Frame_PortalHeading",frame);
	frame.PortalHeading:SetPoint( "BOTTOMLEFT", MagePorts.PortalButtons[1], "TOPLEFT", -cellGap , cellGap);
	frame.PortalHeading:SetPoint( "BOTTOMRIGHT", MagePorts.PortalButtons[cellCount], "TOPRIGHT", cellGap , cellGap);	
	frame.PortalHeading:SetWidth(frame:GetWidth());
	frame.PortalHeading:SetScript("OnEvent",onPortalReagentEvent);
	frame.PortalHeading:RegisterEvent("PLAYER_LOGIN");
	frame.PortalHeading:RegisterEvent("ITEM_PUSH");
	frame.PortalHeading:RegisterEvent("BAG_UPDATE");
	
   	 -- Create the Portal Title 
   	frame.PortalTitle = frame.PortalHeading:CreateFontString("MagePorts_Frame_PortalTitle", 'BACKGROUND', "GameFontNormal");
	frame.PortalTitle:SetPoint( "LEFT" );
	frame.PortalTitle:SetText("");
	
	-- Create the Portal Reagent Count 
    frame.PortalCount = frame.PortalHeading:CreateFontString("MagePorts_Frame_PortalCount", 'BACKGROUND', "GameFontNormal");
	frame.PortalCount:SetPoint( "RIGHT" );	
	frame.PortalCount:SetText("");  

	frame.PortalHeading:SetHeight(frame.PortalTitle:GetStringHeight() + 5 );
	
	frame.TeleportHeading = CreateFrame("Frame","MagePorts_Frame_TeleportHeading",frame);
	frame.TeleportHeading:SetPoint( "BOTTOMLEFT", MagePorts.TeleportButtons[1], "TOPLEFT", -cellGap , cellGap);
	frame.TeleportHeading:SetPoint( "BOTTOMRIGHT", MagePorts.TeleportButtons[cellCount], "TOPRIGHT", cellGap , cellGap);	
	frame.TeleportHeading:SetWidth(frame:GetWidth());
	frame.TeleportHeading:SetScript("OnEvent",onTeleportReagentEvent);
	frame.TeleportHeading:RegisterEvent("PLAYER_LOGIN");
	frame.TeleportHeading:RegisterEvent("ITEM_PUSH");
	frame.TeleportHeading:RegisterEvent("BAG_UPDATE");

	-- Create the Teleport Title
    frame.TeleportTitle = frame.TeleportHeading:CreateFontString("MagePorts_Frame_TeleportTitle", 'BACKGROUND', "GameFontNormal");
	frame.TeleportTitle:SetPoint( "LEFT" );
	frame.TeleportTitle:SetText("");

	-- Create the Teleport Reagent Count
    frame.TeleportCount = frame.TeleportHeading:CreateFontString("MagePorts_Frame_TeleportCount", 'BACKGROUND', "GameFontNormal");
	frame.TeleportCount:SetPoint( "RIGHT" );	
	frame.TeleportCount:SetText("");  
	
	frame.TeleportHeading:SetHeight(frame.TeleportTitle:GetStringHeight() + 5 );
	
	-- Return this frame for use by the addon
	return frame;

end

-----------------------------------------------------------------------
-- [v1.01.00] Check if we are at a reagent vendor and ask if to restock
-- [v1.01.01] Only restock Rune of Portals if over level 35
-- [v1.01.01] Only restock Rune of Teleportation if over level 20
-- [v1.03.00] Use ReagentDB to handle restocking for customization
-- [v1.03.01] Validate Level rules before testing restocking of reagents
-----------------------------------------------------------------------
local function MagePorts_ReagentRestock(Confirmed)

	local ReagentVendor	= false;
	local ReagentsLow	= false;
	local RestockList	= {};
	
	local PlayerLevel = UnitLevel("player");
	 

	-- Create a list of Reagents to Restock	
	for i,reagent in pairs(ReagentDB) do
		if ( PlayerLevel > ( reagent.level - 1 ) ) then
			local name = GetItemInfo(reagent.item);
			local qty = GetItemCount(reagent.item);
			RestockList[name] = false;
			if ( qty < reagent.max ) then 
				ReagentsLow = true;
				RestockList[name] = true;
			end
		end
	end

	-- If we have (MagePorts_Defaults.MaxRestockQty) of each reagent then no need to continue
	if ( not ReagentsLow ) then return; end
	
	-- Check to see if this is a reagent vendor
	-- If it is and we have confirmed restocking then buy the reagents
	ReagentVendor = false;
	for i = 1, GetMerchantNumItems(), 1 do
		local name, _, price, quantity = GetMerchantItemInfo(i);
		if ( RestockList[name] ) then
			ReagentVendor = true;
			if ( Confirmed ) then
				local count	= GetItemCount(name);
				local restock = 0;	
				for j,reagent in pairs(ReagentDB) do
					local dbName = GetItemInfo(reagent.item);
					if ( dbName == name ) then
						if ( count < reagent.max ) then
							restock = reagent.max - count;
							BuyMerchantItem(i,restock);
							break;
						end
					end
				end
			else
				break;
			end
		end
	end

	-- If this is a reagent vendor then pop up the question to restock
	if ( ReagentVendor ) then
		StaticPopup_Show("RESTOCK_REAGENTS");
	end
	
end

-----------------------------------------------------------------------
-- [v1.01.00] Setup for the reagent restocking dialog
-----------------------------------------------------------------------
StaticPopupDialogs["RESTOCK_REAGENTS"] = {
	text = _L["RESTOCK_REAGENTS"],
	button1 = _L["YES"],
	button2 = _L["NO"],
	OnAccept = function() MagePorts_ReagentRestock(true); end,
	timeout = 0,
	whileDead = 0,
	hideOnEscape = 1
};

-----------------------------------------------------------------------
-- [v1.01.00] OnEvent function for this addon
-----------------------------------------------------------------------
local function MagePorts_OnEvent()
	if event == "LEARNED_SPELL_IN_TAB" then
		MagePorts_UpdateFrame(true);
	elseif event == "PLAYER_ENTERING_WORLD" then
		MagePorts_UpdateFrame(true);
	elseif event == "UNIT_SPELLCAST_START" then		
		if ( MagePorts_ValidateSpell("START", arg1, arg2, arg3) ) then 
			MagePorts_UpdateFrame(true);
		end
	elseif event == "UNIT_SPELLCAST_STOP" then
		if ( MagePorts_ValidateSpell("STOP", arg1, arg2, arg3) ) then
			MagePorts_UpdateFrame(true);
		end
	elseif event == "MERCHANT_SHOW" then
		MagePorts_ReagentRestock();
		MagePorts_UpdateFrame(true);
	elseif event == "MERCHANT_CLOSED" then
		StaticPopup_Hide("RESTOCK_REAGENTS");
		MagePorts_UpdateFrame(true);
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- [v1.02.00] Check Slash Commands
-- [v1.03.03] Reset FontSize based on uiScale.  If adjusting scale and it doesn't reload the ui, after reloading manually use /mp fontsize to reset font size.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function MagePorts_SlashCommands(msg)
        msg = string.lower(msg);
		local args = {};
		for word in string.gmatch(msg,"[^%s]+") do
			table.insert(args,word);
		end
		DefaultsChanged = false;
		if ( args[1] == "notext" ) then
			MagePorts_Defaults.DisplayText = not MagePorts_Defaults.DisplayText;		
			DefaultsChanged = true;
        elseif ( args[1] == "restock" ) then
			if ( args[2] and tonumber(args[2]) >= 0 and tonumber(args[2]) <= 100) then
				MagePorts_Defaults.MaxRestockQty = tonumber(args[2]);
			else
				MagePorts_Defaults.MaxRestockQty = 10;
			end
			DefaultsChanged = true;
		elseif ( args[1] == "autoupdate" ) then
			if ( args[2] and tonumber(args[2]) >= 0 and tonumber(args[2]) <= 60) then
				MagePorts_Defaults.UpdateInterval = tonumber(args[2]);
			else
				MagePorts_Defaults.UpdateInterval = 30;
			end
			DefaultsChanged = true;
		elseif ( args[1] == "fontsize" ) then
			if ( args[2] and tonumber(args[2]) >= 8 and tonumber(args[2]) <= 20) then
				MagePorts_Defaults.FontSize = tonumber(args[2]);
			else
				if ( uiScaling == 0 or uiScale == 1 ) then
					MagePorts_Defaults.FontSize = 10;
				else
					MagePorts_Defaults.FontSize = 14;
				end
			end
			DefaultsChanged = true;
		elseif (args[1] == "alpha-na" ) then
			if ( args[2] and tonumber(args[2]) >= 0.0 and tonumber(args[2]) < 1.0 ) then
				MagePorts_Defaults.NotAvailableAlpha = tonumber(args[2]);
			else
				MagePorts_Defaults.NotAvailableAlpha = 0.1;
			end
			DefaultsChanged = true;
		elseif (args[1] == "alpha-ni" ) then
			if ( args[2] and tonumber(args[2]) >= 0.0 and tonumber(args[2]) <= 1.0 ) then
				MagePorts_Defaults.NotIdealAlpha = tonumber(args[2]);
			else
				MagePorts_Defaults.NotIdealAlpha = 0.5;
			end
			DefaultsChanged = true;
		elseif (args[1] == "alpha-ia" ) then
			if ( args[2] and tonumber(args[2]) >= 0.0 and tonumber(args[2]) <= 1.0 ) then
				MagePorts_Defaults.AvailableAlpha = tonumber(args[2]);
			else
				MagePorts_Defaults.AvailableAlpha = 1.0;
			end
			DefaultsChanged = true;
		else
			DEFAULT_CHAT_FRAME:AddMessage(_L["SLASH_COMMAND_LIST"] .. "( /mageports , /mp)" );
			DEFAULT_CHAT_FRAME:AddMessage(_L["SLASH_RESET"]);
			DEFAULT_CHAT_FRAME:AddMessage("notext " .. _L["NOTEXT_SLASH"]);
			DEFAULT_CHAT_FRAME:AddMessage("restock x    ( x >= 0   , x <= 100, 10  ) - " .. _L["RESTOCK_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("autoupdate x ( x >= 0   , x <= 60 , 30  ) - " .. _L["UPDATE_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("fontsize x   ( x >= 8   , x <= 20 , 10 / 14  ) - " .. _L["FONT_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("alpha-na x   ( x >= 0.0 , x <= 1.0, 0.1 ) - " .. _L["ALPHANA_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("alpha-ni x   ( x >= 0.0 , x <= 1.0, 0.5 ) - " .. _L["ALPHANI_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("alpha-ia x   ( x >= 0.0 , x <= 1.0, 1.0 ) - " .. _L["ALPHAIA_SLASH"] );
		end
		if ( DefaultsChanged ) then MagePorts_UpdateFrame(true); end
end
-----------------------------------------------------------------------------


-----------------------------------------------------------------------------
-- [v1.02.00] Set Up Slash Commands
-----------------------------------------------------------------------------
local function MagePorts_SetUpSlashCommands()
	SLASH_MagePortsCmd1 = '/MagePorts';
	SLASH_MagePortsCmd2 = '/mp';
	SlashCmdList['MagePortsCmd'] = MagePorts_SlashCommands;
end
-----------------------------------------------------------------------------

-----------------------------------------------------------------------
-- [v1.01.00] Create Frame Function called from outside to use this addon
-- [v1.02.00] Added Slash Command functionality
-- [v1.03.00] Added Reagent Database for flexibility
-----------------------------------------------------------------------
function MagePorts_CreateFrame()
	local playerClass, englishClass = UnitClass("player");
	MagePorts.Frame = nil;
	if ( englishClass == "MAGE" ) then
		MagePorts_InitReagentDB();
		MagePorts_InitPortalDB();
		MagePorts_InitTeleportDB();
		MagePorts.Frame = MagePorts_InitFrame();
		MagePorts.Frame:SetScript( "OnEvent", MagePorts_OnEvent );
		MagePorts.Frame:RegisterEvent( "LEARNED_SPELL_IN_TAB" );
		MagePorts.Frame:RegisterEvent( "PLAYER_ENTERING_WORLD" );
		MagePorts.Frame:RegisterEvent( "UNIT_SPELLCAST_START" );
		MagePorts.Frame:RegisterEvent( "UNIT_SPELLCAST_STOP" );
		MagePorts.Frame:RegisterEvent( "MERCHANT_SHOW" );
		MagePorts.Frame:RegisterEvent( "MERCHANT_CLOSED" );
		if ( MagePorts.Frame ) then
			MagePorts_SetUpSlashCommands();
		end	
	end
end


