---------------------------------------------------
-- [v1.01.00] Make sure we have nUI tables
---------------------------------------------------
if not nUI_InfoPanels then nUI_InfoPanels = {}; end

----------------------------------------------------------------
-- [v1.01.00] Localized Functions and Tables to speed things up
----------------------------------------------------------------
local _L = nUI_InfoPanel_MagePorts_L;

---------------------------------------------------
-- [v1.01.00] Allocation of the Window ID (unique)
---------------------------------------------------
nUI_INFOMODE_MAGEPORTS  = 18;
nUI_INFOPANEL_MAGEPORTS = "nUI_InfoPanel_MagePorts";

---------------------------------------------------
-- [v1.01.00] Set the Default values for the Infopanel
---------------------------------------------------
nUI_InfoPanels[nUI_INFOPANEL_MAGEPORTS ] =
{	
	enabled   = true,
	desc      = _L["Info Panel Text"],             -- player friendly name/description of the panel
	label     = _L["Info Panel Label"],	       -- label to use on the panel selection button face
	rotation  = nUI_INFOMODE_MAGEPORTS,            -- index or position this panel appears on/in when clicking the selector button
	full_size = true;                              -- this plugin requires the entire info panel port without the button bag
	
	options  =
	{
		enabled  = true,
	},
};

---------------------------------------------------
-- [v1.01.00] Set Plugin Values
---------------------------------------------------
local plugin    = CreateFrame( "Frame", nUI_INFOPANEL_MAGEPORTS, nUI_Dashboard.Anchor );
plugin.active   = true;

---------------------------------------------------
-- [v1.01.00] Validate Addon is loaded
-- [v1.03.04] Use ADDON_LOADED instead of VARIABLES_LOADED 
-- .......... for nUI 5.06.16 compatibility
---------------------------------------------------
local function onMagePortsEvent()
	if event == "ADDON_LOADED" then
		MagePorts_CreateFrame();
		if ( not MagePorts.Frame ) then
			plugin.active = false;
		end
	end	
end

plugin:SetScript( "OnEvent", onMagePortsEvent );
plugin:RegisterEvent( "ADDON_LOADED" );

---------------------------------------------------
-- [v1.01.00] initPanel routine
---------------------------------------------------
plugin.initPanel = function( container, options )
	plugin.container = container;
	plugin.options   = options;
	if options and options.enabled then
		plugin.setEnabled( true );
	end
end

---------------------------------------------------
-- [v1.01.00] sizeChanged routine
-- [v1.01.03] Changed name of addon frame so as not to clash
-- [v1.02.00] Resize and Reposition according to scale
-- [v1.03.03] No need to reposition text in this file
---------------------------------------------------
plugin.sizeChanged = function( scale, height, width )
	local options  = plugin.options;
	plugin.scale = scale;

	-- Grab addon information and unlock frame for movement
	local MagePortsFrame   = plugin.MagePorts;
	nUI_Movers:lockFrame( MagePortsFrame, false, nil );
	
	-- Resize and Reposition according to scale and size of new container
	MagePortsFrame:SetWidth( width ); 
	MagePortsFrame:SetHeight( height ); 
	
	-- Center Buttons across the container
	local cellCount = #MagePorts.PortalButtons;
	local cellWidth = ( width / cellCount );
	local btnSize = cellWidth * 0.8;
	local btnGap = cellWidth * 0.1;
	local verticalOffsetPortal =   height / 5; 
	local verticalOffsetTeleport = height / 4;

	for i=1,cellCount do
		MagePorts.PortalButtons[i]:SetWidth( btnSize );
		MagePorts.PortalButtons[i]:SetHeight( btnSize );
		MagePorts.TeleportButtons[i]:SetWidth( btnSize );
		MagePorts.TeleportButtons[i]:SetHeight( btnSize );
	end
    
    	local odd = ( cellCount % 2 ) == 1;
    	local midwayLeft = nil;
    	local midwayRight = nil;
    	if ( odd ) then 
		midwayLeft = math.ceil( cellCount / 2 );
	else
		midwayLeft = math.floor( cellCount / 2 );
		midwayRight = midwayLeft + 1;
	end

	-- Reposition Portal Buttons and Fontstrings
	if ( not midwayRight ) then
		MagePorts.PortalButtons[midwayLeft]:SetPoint( "CENTER", plugin.container, "CENTER", btnGap/2, verticalOffsetPortal);
		MagePorts.TeleportButtons[midwayLeft]:SetPoint( "CENTER", plugin.container, "CENTER", btnGap/2, -verticalOffsetTeleport);
	else
	    	MagePorts.PortalButtons[midwayLeft]:SetPoint( "RIGHT", plugin.container, "CENTER", -btnGap/2, verticalOffsetPortal );
		MagePorts.PortalButtons[midwayRight]:SetPoint( "LEFT", plugin.container, "CENTER", btnGap/2, verticalOffsetPortal );
	    	MagePorts.TeleportButtons[midwayLeft]:SetPoint( "RIGHT", plugin.container, "CENTER", -btnGap/2, -verticalOffsetTeleport );
		MagePorts.TeleportButtons[midwayRight]:SetPoint( "LEFT", plugin.container, "CENTER", btnGap/2, -verticalOffsetTeleport );
	end
	for i = 1, midwayLeft - 1 do	
		MagePorts.PortalButtons[i]:SetPoint( "RIGHT", MagePorts.PortalButtons[i+1], "LEFT", -btnGap, 0 );
		MagePorts.TeleportButtons[i]:SetPoint( "RIGHT", MagePorts.TeleportButtons[i+1], "LEFT", -btnGap, 0 );
	end
	for i = midwayLeft + 1, cellCount do	
		MagePorts.PortalButtons[i]:SetPoint( "LEFT", MagePorts.PortalButtons[i-1], "RIGHT", btnGap, 0 );
		MagePorts.TeleportButtons[i]:SetPoint( "LEFT", MagePorts.TeleportButtons[i-1], "RIGHT", btnGap, 0 );
	end
	
	-- Lock addon frame for movement
	nUI_Movers:lockFrame( MagePortsFrame, true, nil );
end	

---------------------------------------------------
-- [v1.01.00] Validate Enable Status
-- [v1.03.03] Shrunk Plugin Frame slightly so as to leave a border gap
---------------------------------------------------
plugin.setEnabled = function( enabled )
	if plugin.enabled ~= enabled then
		plugin.enabled = enabled;
		if not enabled then
			local MagePorts = plugin.MagePorts;
			if MagePorts.saved_parent then
				nUI_Movers:lockFrame( MagePorts, false, nil );
				MagePorts:SetParent( MagePorts.saved_parent );
				MagePorts:SetBackdropBorderColor( MagePorts.border_color );
				MagePorts:SetBackdropColor( MagePorts.backdrop_color );
			end
		else
			local MagePorts = MagePorts.Frame;
			plugin.MagePorts = MagePorts;
			if not MagePorts.saved_parent then
				MagePorts.saved_parent   = MagePorts:GetParent();
				MagePorts.border_color   = MagePorts:GetBackdropBorderColor();
				MagePorts.backdrop_color = MagePorts:GetBackdropColor();
			end
			MagePorts:SetParent( plugin.container );
			MagePorts:SetPoint( "TOPLEFT", plugin.container, "TOPLEFT", 5, 5 );
			MagePorts:SetPoint( "BOTTOMRIGHT", plugin.container, "BOTTOMRIGHT", -5, -5 );
			MagePorts:SetFrameStrata( plugin.container:GetFrameStrata() );
			MagePorts:SetFrameLevel( plugin.container:GetFrameLevel()+1 );
			MagePorts:SetBackdropBorderColor( 0, 0, 0, 0 );
			MagePorts:SetBackdropColor( 0, 0, 0, 0 );
			
			nUI_Movers:lockFrame( MagePorts, true, nil );
		end				
	end			
end

---------------------------------------------------
-- [v1.01.00] setSelected routine
---------------------------------------------------
plugin.setSelected = function( selected )
	if selected ~= plugin.selected then
		plugin.selected = selected;
		if selected then
		else
		end
	end
end
